#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2016 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

from b3j0f.utils.ut import UTCase

from unittest import main

from ..core import getprotocols, unregister
from ..cls import Middleware


class ClsTest(UTCase):
    """Test for the functions register and unregister."""

    def setUp(self):

        unregister()

    def test_cls(self):

        class A(Middleware):

            __protocols__ = ['a']

        class BC(A):

            __protocols__ = ['b', 'c']

        def assertcls(cls, protocols):

            clsprotocols = cls.protocols()
            self.assertEqual(clsprotocols, set(protocols))

            clsprotocols = getprotocols(cls)
            self.assertEqual(clsprotocols, set(protocols))

        assertcls(A, 'a')
        assertcls(BC, ['a', 'b', 'c'])

    def tearDown(self):

        unregister()


if __name__ == '__main__':
    main()
